#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAXSTR 255

typedef struct {
    char *nome;
    float distancia;
} TVPlaneta;

typedef struct {
    char *estrela;
    TVPlaneta *planeta;
} TSPlanetario;

void AlocaMem(TSPlanetario *sistemap, int i, char *str, char *estrutura) {

    if (strcmp(estrutura, "estrela") == 0) {
        sistemap->estrela = (char *) malloc((strlen(str) + 1) * sizeof(char));
    }

    if (strcmp(estrutura, "planeta") == 0) {
        if (sistemap->planeta == NULL) {
            sistemap->planeta = (TVPlaneta *) malloc(((i + 1) * sizeof(TVPlaneta)));
        } else {
            sistemap->planeta = (TVPlaneta *) realloc(sistemap->planeta, ((i + 1) * sizeof(TVPlaneta)));
        }
        sistemap->planeta[i].nome = (char *) malloc((strlen(str) + 1) * sizeof(char));
    }
}

TSPlanetario SPCriar() {
    TSPlanetario sistemap;
    sistemap.estrela = NULL;
    sistemap.planeta = NULL;
    return sistemap;
}

void SPLeEstrela(TSPlanetario *sistemap) {
    char str[MAXSTR], provisorio[MAXSTR];

    fgets(provisorio, MAXSTR, stdin);
    sscanf(provisorio, "%[^\n, ,]", str);
    AlocaMem(sistemap, (int) 0, str, "estrela");

    if (sistemap->estrela != NULL) {
        strcpy(sistemap->estrela, str);
    }
}

void SPLePlaneta(TSPlanetario *sistemap, int i) {
    char str[MAXSTR], provisorio[MAXSTR];
    float distancia;

    *str = '\0';
    distancia = 0;
    fgets(provisorio, MAXSTR, stdin);
    sscanf(provisorio, "%[^\n, ,0-9] %f", str, &distancia);
    AlocaMem(sistemap, i, str, "planeta");

    if (sistemap->planeta != NULL && sistemap->planeta[i].nome != NULL) {
        strcpy(sistemap->planeta[i].nome, str);
        sistemap->planeta[i].distancia = distancia;
    }
}

int ContaPlaneta(TSPlanetario *sistemap) {
    int i, conta;

    i = 0;
    conta = 0;
    while ((strcmp(sistemap->planeta[i].nome, "\0") != 0)) {
        i++;
        conta++;
    }
    return conta;
}

void OrdenaPlaneta(TSPlanetario *sistemap, int tamanho) {
    int i, j;
    TSPlanetario aux;

    aux.planeta = (TVPlaneta *) malloc(((1) * sizeof(TVPlaneta)));

    if (aux.planeta != NULL) {
        for (i = 0; i < tamanho; i++)
            for (j = i + 1; j < tamanho; j++)
                if (sistemap->planeta[i].distancia > sistemap->planeta[j].distancia) {
                    aux.planeta[0].nome = (char *) malloc((strlen(sistemap->planeta[i].nome) + 1) * sizeof(char));
                    aux.planeta[0] = sistemap->planeta[i];
                    sistemap->planeta[i] = sistemap->planeta[j];
                    sistemap->planeta[j] = aux.planeta[0];
                }
    }
    free(aux.planeta);
}

void SPLibertar(TSPlanetario *sistemap) {
    int i, tamanho;

    if (sistemap->estrela != NULL) {
        free(sistemap->estrela);
        sistemap->estrela = NULL;
    }

    tamanho = ContaPlaneta(sistemap);
    for (i = 0; i <= tamanho; i++) {
        if (sistemap->planeta[i].nome != NULL) {
            free(sistemap->planeta[i].nome);
            sistemap->planeta[i].nome = NULL;
            sistemap->planeta[i].distancia = 0;
        }
    }

    if (sistemap->planeta != NULL) {
        free(sistemap->planeta);
        sistemap->planeta = NULL;
    }
}

/*---------- Programa ----------*/

TSPlanetario LerSistema() {
    TSPlanetario sistemap;
    int i;

    sistemap = SPCriar();

    SPLeEstrela(&sistemap);

    i = 0;
    do {
        SPLePlaneta(&sistemap, i);
        i++;
    } while ((strcmp(sistemap.planeta[i - 1].nome, "\0") != 0));

    OrdenaPlaneta(&sistemap, ContaPlaneta(&sistemap));
    return sistemap;
}

void MostraSistema(TSPlanetario *sistemap) {
    printf("%s, sistema planetário com %d planetas a distâncias entre %.2f e %.2f UA.",
           sistemap->estrela,
           ContaPlaneta(sistemap),
           sistemap->planeta[0].distancia,
           sistemap->planeta[ContaPlaneta(sistemap) - 1].distancia);
}

void main() {
    TSPlanetario sistemap;
    sistemap = LerSistema();
    MostraSistema(&sistemap);
    SPLibertar(&sistemap);
    exit(0);
}