#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#define MAXSTR 255

typedef struct {
    char *nome;
    float distancia;
} TVSatelite;

typedef struct {
    char *nome;
    float distancia;
    TVSatelite *satelite;
} TVPlaneta;

typedef struct {
    char *estrela;
    TVPlaneta *planeta;
} TSPlanetario;

typedef struct {
    char *astro;
    char *origem;
    int indexi;
    int indexj;
} TOrigem;

typedef struct {
    char *astro;
    char *destino;
    int indexi;
    int indexj;
} TDestino;

typedef struct {
    char *astro;
    float distancia;
} TRota;

typedef struct {
    TOrigem origem;
    TDestino destino;
    TRota *rota;
    float distancia;
} TViagem;

TViagem VCriar() {
    TViagem viagem;
    viagem.rota = NULL;
    viagem.origem.origem = NULL;
    viagem.destino.destino = NULL;
    return viagem;
}

TSPlanetario SPCriar() {
    TSPlanetario sistemap;
    sistemap.estrela = NULL;
    sistemap.planeta = NULL;
    return sistemap;
}

void AlocaMem(TSPlanetario *sistemap, TViagem *viagem, int i, int j, char *str, char *estrutura) {
    if (strcmp(estrutura, "estrela") == 0) {
        sistemap->estrela = (char *) malloc((strlen(str) + 1) * sizeof(char));
    }

    if (strcmp(estrutura, "planeta") == 0) {
        if (sistemap->planeta == NULL) {
            sistemap->planeta = (TVPlaneta *) malloc(((i + 1) * sizeof(TVPlaneta)));
        } else {
            sistemap->planeta = (TVPlaneta *) realloc(sistemap->planeta, ((i + 1) * sizeof(TVPlaneta)));
        }
        sistemap->planeta[i].nome = (char *) malloc((strlen(str) + 1) * sizeof(char));
    }

    if (strcmp(estrutura, "satelite") == 0) {
        if (sistemap->planeta[i].satelite == NULL) {
            sistemap->planeta[i].satelite = (TVSatelite *) malloc(((j + 1) * sizeof(TVSatelite)));
        } else {
            sistemap->planeta[i].satelite = (TVSatelite *) realloc(sistemap->planeta[i].satelite,
                                                                   ((j + 1) * sizeof(TVSatelite)));
        }
        sistemap->planeta[i].satelite[j].nome = (char *) malloc((strlen(str) + 1) * sizeof(char));
    }

    if (strcmp(estrutura, "viagem") == 0) {
        if (viagem->rota == NULL) {
            viagem->rota = (TRota *) malloc(((i + 1) * sizeof(TRota)));
        } else {
            viagem->rota = (TRota *) realloc(viagem->rota, ((i + 1) * sizeof(TRota)));
        }
        viagem->rota[i].astro = (char *) malloc((strlen(str) + 1) * sizeof(char));
    }
}

void SPLeEstrela(TSPlanetario *sistemap) {
    char str[MAXSTR], provisorio[MAXSTR];

    fgets(provisorio, MAXSTR, stdin);
    sscanf(provisorio, "%[^\n, ,]", str);
    AlocaMem(sistemap, 0, (int) 0, (int) 0, str, "estrela");

    if (sistemap->estrela != NULL) {
        strcpy(sistemap->estrela, str);
    }
}

void SPLePlaneta(TSPlanetario *sistemap, int i, char *provisorio) {
    char str[MAXSTR];
    float distancia;

    *str = '\0';
    distancia = 0;
    sscanf(provisorio, "%[^\n, ,] %f", str, &distancia);
    AlocaMem(sistemap, 0, i, (int) 0, str, "planeta");

    if (sistemap->planeta != NULL && sistemap->planeta[i].nome != NULL) {
        strcpy(sistemap->planeta[i].nome, str);
        sistemap->planeta[i].distancia = distancia;
    }
}

void SPLeSatelite(TSPlanetario *sistemap, int i, int j, char *provisorio) {
    char strs[MAXSTR];
    float distancia;

    *strs = '\0';
    distancia = 0;
    sscanf(provisorio, "%*c %[^\n, ] %f", strs, &distancia);
    AlocaMem(sistemap, 0, i, j, strs, "satelite");

    if (sistemap->planeta[i].satelite != NULL && sistemap->planeta[i].satelite[j].nome != NULL) {
        strcpy(sistemap->planeta[i].satelite[j].nome, strs);
        sistemap->planeta[i].satelite[j].distancia = distancia;
    }
}

void SPLeSistema(TSPlanetario *sistemap) {
    char provisorio[MAXSTR];
    int iplaneta, isatelite;

    iplaneta = 0;
    do {
        fgets(provisorio, MAXSTR, stdin);
        if (provisorio[0] == '#') {
            SPLeSatelite(sistemap, iplaneta - 1, isatelite, provisorio);
            SPLeSatelite(sistemap, iplaneta - 1, isatelite + 1, "");
            isatelite++;
        } else {
            isatelite = 0;
            SPLePlaneta(sistemap, iplaneta, provisorio);
            sistemap->planeta[iplaneta].satelite = NULL;
            SPLeSatelite(sistemap, iplaneta, isatelite, "");
            iplaneta++;
        }
    } while ((strcmp(sistemap->planeta[iplaneta - 1].nome, "\0") != 0));
}

int ContaAstros(TSPlanetario *sistemap, TViagem *viagem, int i, char *estrutura) {
    int j, conta;

    conta = 0;
    if (strcmp(estrutura, "planeta") == 0) {
        while ((strcmp(sistemap->planeta[i].nome, "\0") != 0)) {
            i++;
            conta++;
        }
    }

    if (strcmp(estrutura, "satelite") == 0) {
        j = 0;
        while ((strcmp(sistemap->planeta[i].satelite[j].nome, "\0") != 0)) {
            j++;
            conta++;
        }
    }

    if (strcmp(estrutura, "viagem") == 0) {
        while ((strcmp(viagem->rota[i].astro, "\0") != 0)) {
            i++;
            conta++;
        }
    }
    return conta;
}

void VProcessaOrigemA(TViagem *viagem, char *str, int i, int j) {
    viagem->origem.astro = (char *) malloc((strlen(str) + 1) * sizeof(char));
    strcpy(viagem->origem.astro, str);
    viagem->origem.indexi = i;
    viagem->origem.indexj = j;
}

void VProcessaOrigem(TSPlanetario *sistemap, TViagem *viagem, char *origem) {
    int i, j, tplaneta, tsatelite;

    VProcessaOrigemA(viagem, "estrela", 0, 0);

    tplaneta = ContaAstros(sistemap, 0, 0, "planeta");
    for (i = 0; i < tplaneta; i++) {
        if (strcmp(sistemap->planeta[i].nome, origem) == 0) {
            VProcessaOrigemA(viagem, "planeta", i, 0);
        } else {
            tsatelite = ContaAstros(sistemap, 0, i, "satelite");
            for (j = 0; j < tsatelite; j++) {
                if (strcmp(sistemap->planeta[i].satelite[j].nome, origem) == 0) {
                    VProcessaOrigemA(viagem, "satelite", i, j);
                }
            }
        }
    }
}

void VProcessaDestinoA(TViagem *viagem, char *str, int i, int j) {
    viagem->destino.astro = (char *) malloc((strlen(str) + 1) * sizeof(char));
    strcpy(viagem->destino.astro, str);
    viagem->destino.indexi = i;
    viagem->destino.indexj = j;
}

void VProcessaDestino(TSPlanetario *sistemap, TViagem *viagem, char *destino) {
    int i, j, tplaneta, tsatelite;

    VProcessaDestinoA(viagem, "estrela", 0, 0);
    tplaneta = ContaAstros(sistemap, 0, 0, "planeta");
    for (i = 0; i < tplaneta; i++) {
        if (strcmp(sistemap->planeta[i].nome, destino) == 0) {
            VProcessaDestinoA(viagem, "planeta", i, 0);
        } else {
            tsatelite = ContaAstros(sistemap, 0, i, "satelite");
            for (j = 0; j < tsatelite; j++) {
                if (strcmp(sistemap->planeta[i].satelite[j].nome, destino) == 0) {
                    VProcessaDestinoA(viagem, "satelite", i, j);
                }
            }
        }
    }
}

void VProcessaRotaOrigemA(TSPlanetario *sistemap, TViagem *viagem, char *str) {
    if (viagem->origem.origem == NULL) {
        viagem->origem.origem = (char *) malloc((strlen(sistemap->estrela) + 1) * sizeof(char));
        strcpy(viagem->origem.origem, sistemap->estrela);
    } else {
        viagem->origem.origem = (char *) realloc(viagem->origem.origem,
                                                 ((strlen(viagem->origem.origem) + strlen(str)) + 1) * sizeof(char));
        strcat(viagem->origem.origem, str);
    }
}

int VProcessaRotaOrigemP(TSPlanetario *sistemap, TViagem *viagem, int irota) {
    int auxplaneta;
    AlocaMem(0, viagem, irota, 0, sistemap->planeta[viagem->origem.indexi].nome, "viagem");
    strcpy(viagem->rota[irota].astro, sistemap->planeta[viagem->origem.indexi].nome);
    viagem->rota[irota].distancia = ceil(((sistemap->planeta[viagem->origem.indexi].distancia) * 30));
    viagem->distancia += viagem->rota[irota].distancia;
    VProcessaRotaOrigemA(sistemap, viagem, sistemap->planeta[viagem->origem.indexi].nome);
    irota++;
    auxplaneta = irota;
    return auxplaneta;
}

int VProcessaRotaOrigemS(TSPlanetario *sistemap, TViagem *viagem, int irota) {
    int auxsatelite;

    AlocaMem(0, viagem, irota, 0, sistemap->planeta[viagem->origem.indexi].satelite[viagem->origem.indexj].nome,
             "viagem");
    strcpy(viagem->rota[irota].astro, sistemap->planeta[viagem->origem.indexi].satelite[viagem->origem.indexj].nome);
    viagem->rota[irota].distancia = ceil(
            ((sistemap->planeta[viagem->origem.indexi].satelite[viagem->origem.indexj].distancia) * 30));
    viagem->distancia += viagem->rota[irota].distancia;
    irota++;

    auxsatelite = VProcessaRotaOrigemP(sistemap, viagem, irota);
    VProcessaRotaOrigemA(sistemap, viagem, "-");
    VProcessaRotaOrigemA(sistemap, viagem,
                         sistemap->planeta[viagem->origem.indexi].satelite[viagem->origem.indexj].nome);
    return auxsatelite;
}

int VProcessaRotaOrigem(TSPlanetario *sistemap, TViagem *viagem) {
    int irota;
    irota = 0;

    viagem->distancia = 0;
    VProcessaRotaOrigemA(sistemap, viagem, sistemap->estrela);
    VProcessaRotaOrigemA(sistemap, viagem, "-");

    if (strcmp(viagem->origem.astro, "satelite") == 0) {
        irota = VProcessaRotaOrigemS(sistemap, viagem, irota);
    }

    if (strcmp(viagem->origem.astro, "planeta") == 0) {
        irota = VProcessaRotaOrigemP(sistemap, viagem, irota);
    }

    return irota;
}

void VProcessaRotaDestinoA(TSPlanetario *sistemap, TViagem *viagem, char *str) {
    if (viagem->destino.destino == NULL) {
        viagem->destino.destino = (char *) malloc((strlen(sistemap->estrela) + 1) * sizeof(char));
        strcpy(viagem->destino.destino, sistemap->estrela);
    } else {
        viagem->destino.destino = (char *) realloc(viagem->destino.destino,
                                                   ((strlen(viagem->destino.destino) + strlen(str)) + 1) *
                                                   sizeof(char));
        strcat(viagem->destino.destino, str);
    }
}

int VProcessaRotaDestinoP(TSPlanetario *sistemap, TViagem *viagem, int irota) {
    int auxplaneta;
    AlocaMem(0, viagem, irota, 0, sistemap->estrela, "viagem");
    strcpy(viagem->rota[irota].astro, sistemap->estrela);
    viagem->rota[irota].distancia = ceil(((sistemap->planeta[viagem->destino.indexi].distancia) * 30));
    viagem->distancia += viagem->rota[irota].distancia;
    irota++;

    AlocaMem(0, viagem, irota, 0, sistemap->planeta[viagem->destino.indexi].nome, "viagem");
    strcpy(viagem->rota[irota].astro, sistemap->planeta[viagem->destino.indexi].nome);
    viagem->rota[irota].distancia = 0;
    irota++;
    auxplaneta = irota;
    return auxplaneta;
}

int VProcessaRotaDestinoS(TSPlanetario *sistemap, TViagem *viagem, int irota) {
    int auxsatelite;

    auxsatelite = irota;
    if (viagem->origem.indexi != viagem->destino.indexi) {
        auxsatelite = VProcessaRotaDestinoP(sistemap, viagem, irota);
    }

    AlocaMem(0, viagem, auxsatelite, 0, sistemap->planeta[viagem->destino.indexi].satelite[viagem->destino.indexj].nome,
             "viagem");
    strcpy(viagem->rota[auxsatelite].astro,
           sistemap->planeta[viagem->destino.indexi].satelite[viagem->destino.indexj].nome);
    viagem->distancia -= viagem->rota[auxsatelite - 1].distancia;
    viagem->rota[auxsatelite - 1].distancia = ceil(
            ((sistemap->planeta[viagem->destino.indexi].satelite[viagem->destino.indexj].distancia) * 30));
    viagem->rota[auxsatelite].distancia = 0;
    viagem->distancia += viagem->rota[auxsatelite - 1].distancia;
    auxsatelite++;

    VProcessaRotaDestinoA(sistemap, viagem, sistemap->planeta[viagem->destino.indexi].nome);
    VProcessaRotaDestinoA(sistemap, viagem, "-");
    VProcessaRotaDestinoA(sistemap, viagem,
                          sistemap->planeta[viagem->destino.indexi].satelite[viagem->destino.indexj].nome);
    return auxsatelite;
}

void VProcessaRotaDestino(TSPlanetario *sistemap, TViagem *viagem, int irota) {
    VProcessaRotaDestinoA(sistemap, viagem, sistemap->estrela);
    VProcessaRotaDestinoA(sistemap, viagem, "-");

    if (strcmp(viagem->destino.astro, "satelite") == 0) {
        irota = VProcessaRotaDestinoS(sistemap, viagem, irota);
    }

    if (strcmp(viagem->destino.astro, "planeta") == 0) {
        irota = VProcessaRotaDestinoP(sistemap, viagem, irota);
        VProcessaRotaDestinoA(sistemap, viagem, sistemap->planeta[viagem->destino.indexi].nome);
    }
    AlocaMem(0, viagem, irota, 0, "\0", "viagem");
    strcpy(viagem->rota[irota].astro, "\0");
}

void VProcessaRota(TSPlanetario *sistemap, TViagem *viagem) {
    int irota;
    irota = VProcessaRotaOrigem(sistemap, viagem);
    VProcessaRotaDestino(sistemap, viagem, irota);
}

void VProcessaEstacoes(TSPlanetario *sistemap, TViagem *viagem) {
    char provisorio[MAXSTR], origem[MAXSTR], destino[MAXSTR];
    fgets(provisorio, MAXSTR, stdin);
    sscanf(provisorio, "%[^\n, ] %[^\n, ]", origem, destino);
    VProcessaOrigem(sistemap, viagem, origem);
    VProcessaDestino(sistemap, viagem, destino);
    VProcessaRota(sistemap, viagem);

}


void SPLibertar(TSPlanetario *sistemap, TViagem *viagem) {
    int i, j, tplaneta, tsatelite, trota;

    if (sistemap->estrela != NULL) {
        free(sistemap->estrela);
        sistemap->estrela = NULL;
    }

    tplaneta = ContaAstros(sistemap, 0, 0, "planeta");
    for (i = 0; i <= tplaneta; i++) {
        tsatelite = ContaAstros(sistemap, 0, i, "satelite");
        for (j = 0; j <= tsatelite; j++) {
            if (sistemap->planeta[i].satelite[j].nome != NULL) {
                free(sistemap->planeta[i].satelite[j].nome);
                sistemap->planeta[i].satelite[j].nome = NULL;
                sistemap->planeta[i].satelite[j].distancia = 0;
            }
        }

        if (sistemap->planeta[i].satelite != NULL) {
            free(sistemap->planeta[i].satelite);
            sistemap->planeta[i].satelite = NULL;
        }

        if (sistemap->planeta[i].nome != NULL) {
            free(sistemap->planeta[i].nome);
            sistemap->planeta[i].nome = NULL;
            sistemap->planeta[i].distancia = 0;
        }
    }

    if (sistemap->planeta != NULL) {
        free(sistemap->planeta);
        sistemap->planeta = NULL;
    }

    trota = ContaAstros(0, viagem, 0, "viagem");
    for (i = 0; i <= trota; i++) {
        if (viagem->rota[i].astro != NULL) {
            free(viagem->rota[i].astro);
            viagem->rota[i].astro = NULL;
            viagem->rota[i].distancia = 0;
        }
    }

    if (viagem->origem.astro != NULL) {
        free(viagem->origem.astro);
        free(viagem->origem.origem);
        viagem->origem.astro = NULL;
        viagem->origem.origem = NULL;
        viagem->origem.indexi = 0;
        viagem->origem.indexj = 0;
    }

    if (viagem->destino.astro != NULL) {
        free(viagem->destino.astro);
        free(viagem->destino.destino);
        viagem->destino.astro = NULL;
        viagem->destino.destino = NULL;
        viagem->destino.indexi = 0;
        viagem->destino.indexj = 0;
    }
}

TSPlanetario ProcessaSistema() {
    TSPlanetario sistemap;

    sistemap = SPCriar();
    SPLeEstrela(&sistemap);
    SPLeSistema(&sistemap);
    return sistemap;
}

TViagem ProcessaViagem(TSPlanetario *sistemap) {
    TViagem viagem;

    viagem = VCriar();
    VProcessaEstacoes(sistemap, &viagem);
    return viagem;
}

void MostraSistema(TSPlanetario *sistemap, TViagem *viagem) {
    int i, trota, auxdistancia, auxdias, dias[] = {360, 30, 1, 0};
    char *singular[] = {"ano", "mês", "dia"}, *plural[] = {"anos", "meses", "dias"};
    auxdistancia = (int) viagem->distancia + 2;
    trota = ContaAstros(0, viagem, 0, "viagem");

    printf("De: %s\n", viagem->origem.origem);
    printf("Para: %s\n", viagem->destino.destino);
    printf("Estações:");
    for (i = 0; i < trota; i++) {
        printf(" %s", viagem->rota[i].astro);
        if (viagem->rota[i].distancia > 0) {
            printf(" (%d)", (int) viagem->rota[i].distancia);
        }
    }

    printf("\nDuração: ");
    for (i = 0; dias[i] > 0; i++) {
        auxdias = (int) (auxdistancia / dias[i]);
        auxdistancia -= dias[i] * auxdias;
        if (auxdias > 0 && auxdias <= 1) {
            printf(" %d %s", auxdias, singular[i]);
        }
        if (auxdias > 1) {
            printf(" %d %s", auxdias, plural[i]);
        }
    }
}

void main() {
    TSPlanetario sistemap;
    TViagem viagem;
    sistemap = ProcessaSistema();
    viagem = ProcessaViagem(&sistemap);
    MostraSistema(&sistemap, &viagem);
    SPLibertar(&sistemap, &viagem);
    exit(0);
}